#include "bitmap.h"
#include "player.h"
#include "random.h"
#include "pad.h"
#include "coll.h"

int col_maxi = 255;

Uint8 bm_pixel_get_scroll(Uint8 scroll, int x, int y){
  Uint8 col;
  switch (scroll) {
  case bmNBG0 : col = bm_pixel_get2(x,y); break;
  case bmNBG1 : col = bm_pixel_get(x,y);  break;
  case bmRBG0 : col = bm_pixel_get3(x,y); break;
  }
  return col;
}

void bm_pixel_set_scroll(Uint8 scroll, int x, int y,Uint8 v){
  switch (scroll) {
  case bmNBG0 : bm_pixel_set2(x,y,v); break;
  case bmNBG1 : bm_pixel_set(x,y,v); break;
  case bmRBG0 : bm_pixel_set3(x,y,v); break;
  }
}

Uint8 bm_pixel_get(int x, int y){
  Uint8 v;
  slBMGet(x,y,x,y,&v);
  return v;
}
void bm_pixel_set(int x, int y, Uint8 v){
  slBMPset(x,y,v);
}

void bm_pixel_set2(int x,int y, Uint8 col){
  Sint8* where = slLocateBitMap(x,y);
  if (where){
    *(where+(NBG1_CEL_ADR-NBG0_CEL_ADR)) =  col;
  }
}

void bm_pixel_set3(int x,int y, Uint8 col){
  Sint8* where = slLocateBitMap(x,y);
  if (where){
    *(where+(RBG_CEL_ADR-NBG1_CEL_ADR)) =  col;
  }
}


Uint8 bm_pixel_get2(int x,int y){
  Sint8* where = slLocateBitMap(x,y);
  if (where){
    Uint8 col =  *(where+(NBG1_CEL_ADR-NBG0_CEL_ADR));
    return col;
  }
  return 0;
}

Uint8 bm_pixel_get3(int x,int y){
  Sint8* where = slLocateBitMap(x,y);
  if (where){
    Uint8 col =  *(where+(RBG_CEL_ADR-NBG0_CEL_ADR));
    return col;
  }
  return 0;
}


void bm_rect_f(Uint8 bitmap,int x,int y, int dx, int dy, int col){
  int i,j;
  for (i=x;i<(x+dx);i++){
    for (j=y;j<(y+dy);j++){
      bm_pixel_set_scroll(bitmap,i,j,col);
    }
  }
}


//
// Palette
//

Uint16	pal_vf[] = {

  0x801f,0xfc1c,0xfc03,0xfee0,0xbfe0,0x83ee,0x83fb,0x819f,
  0x801f,0xffff,0xffff,0xffff,0xffff,0xffff,0xffff,0xffff,
  0xffff,0xfbde,0xfbde,0xfbde,0xfbde,0xfbde,0xfbde,0xfbde,
  0xf7bd,0xf7bd,0xf7bd,0xf7bd,0xf7bd,0xf7bd,0xf7bd,0xf7bd,
  0xf39c,0xf39c,0xf39c,0xf39c,0xf39c,0xf39c,0xf39c,0xf39c,
  0xef7b,0xef7b,0xef7b,0xef7b,0xef7b,0xef7b,0xef7b,0xef7b,
  0xeb5a,0xeb5a,0xeb5a,0xeb5a,0xeb5a,0xeb5a,0xeb5a,0xe739,
  0xe739,0xe739,0xe739,0xe739,0xe739,0xe739,0xe739,0xe318,
  0xe318,0xe318,0xe318,0xe318,0xe318,0xe318,0xe318,0xdef7,
  0xdef7,0xdef7,0xdef7,0xdef7,0xdef7,0xdef7,0xdad6,0xdad6,
  0xdad6,0xdad6,0xdad6,0xdad6,0xdad6,0xdad6,0xd6b5,0xd6b5,
  0xd6b5,0xd6b5,0xd6b5,0xd6b5,0xd6b5,0xd6b5,0xd294,0xd294,
  0xd294,0xd294,0xd294,0xd294,0xd294,0xd294,0xce73,0xce73,
  0xce73,0xce73,0xce73,0xce73,0xce73,0xca52,0xca52,0xca52,
  0xca52,0xca52,0xca52,0xca52,0xca52,0xc631,0xc631,0xc631,
  0xc631,0xc631,0xc631,0xc631,0xc631,0xc210,0xc210,0xc210,
  0xc210,0xc210,0xc210,0xc210,0xc210,0xbdef,0xbdef,0xbdef,
  0xbdef,0xbdef,0xbdef,0xbdef,0xb9ce,0xb9ce,0xb9ce,0xb9ce,
  0xb9ce,0xb9ce,0xb9ce,0xb9ce,0xb5ad,0xb5ad,0xb5ad,0xb5ad,
  0xb5ad,0xb5ad,0xb5ad,0xb5ad,0xb18c,0xb18c,0xb18c,0xb18c,
  0xb18c,0xb18c,0xb18c,0xad6b,0xad6b,0xad6b,0xad6b,0xad6b,
  0xad6b,0xad6b,0xad6b,0xa94a,0xa94a,0xa94a,0xa94a,0xa94a,
  0xa94a,0xa94a,0xa94a,0xa529,0xa529,0xa529,0xa529,0xa529,
  0xa529,0xa529,0xa529,0xa108,0xa108,0xa108,0xa108,0xa108,
  0xa108,0xa108,0x9ce7,0x9ce7,0x9ce7,0x9ce7,0x9ce7,0x9ce7,
  0x9ce7,0x9ce7,0x98c6,0x98c6,0x98c6,0x98c6,0x98c6,0x98c6,
  0x98c6,0x98c6,0x94a5,0x94a5,0x94a5,0x94a5,0x94a5,0x94a5,
  0x94a5,0x9084,0x9084,0x9084,0x9084,0x9084,0x9084,0x9084,
  0x9084,0x8c63,0x8c63,0x8c63,0x8c63,0x8c63,0x8c63,0x8c63,
  0x8c63,0x8842,0x8842,0x8842,0x8842,0x8842,0x8842,0x8842,
  0x8842,0x8421,0x8421,0x8421,0x8421,0x8421,0x8421,0x8421,
  0x8000,0x8000,0x8000,0x8000,0x8000,0x8000,0x8000,0x8000,
};

void bitmap_init(){
  slDMACopy((void*)S2D_PAL, (void*)NBG1_COL_ADR, (Uint32)S2D_PALSIZE);
 
  slInitBitMap(bmRBG0, BM_512x256,(void *)RBG_CEL_ADR);
  slClearBitMap(0);
  slInitBitMap(bmNBG0, BM_512x256,(void *)NBG0_CEL_ADR);
  slClearBitMap(0);
  slInitBitMap(bmNBG1, BM_512x256,(void *)NBG1_CEL_ADR);
  slClearBitMap(0);


  // BM_512x256
 
  slBMPaletteNbg0(0);
  slBMPaletteNbg1(0);
  slBMPaletteRbg0(0);
 
  slScrPosNbg0(toFIXED(0.0),toFIXED(0.0)); 
  // slScrPosRbg0(toFIXED(0.0),toFIXED(0.0)); 
  slScrPosNbg1(toFIXED(0.0),toFIXED(0.0)); 
 
  /* 
     slColorCalc :
     
     Calculation method:                       [CC_RATE| CC_ADD ]|
     Image for which calculation is specified: [CC_TOP | CC_2ND ]|
     Extended color calculations:              [CC_EXT ]|
     Registered screen:                        [NBG0|NBG1|NBG2|NBG3|RBG0|LNCLON|SPRON]|
  */
 
  slColorCalc(CC_RATE| CC_TOP |NBG0ON | NBG1ON );
  
  slColorCalcOn(NBG1ON | NBG0ON );

  slColRateNbg0(15);
  slColRateNbg1(5);
  slColRateRbg0(12);
  /*
    slSpecialCCalcBitNbg0
    slSpecialCCalcModeNbg0(Uint16) ;
    slSpecialPrioBitNbg0(flag)	
  */
  // slScrTransparent(NBG1ON|NBG0ON|RBG0ON);
  slScrTransparent(0);

  slPriorityNbg1(1);
  slPrioritySpr0(2);
  slPriorityNbg0(3);
  slPriorityRbg0(4);


  slScrAutoDisp(NBG1ON|NBG0ON);//|RBG0ON);


  //  slBMBoxFill(-X_MAX/2,-Y_MAX/2,X_MAX/2,Y_MAX/2 ,PAL_T);
}



TANK_COLOR tank_cols[] = {
  {C_RGB(20,0,0),C_RGB(31,0,0),C_RGB(14,0,0),C_RGB(5,0,0),C_RGB(10,0,0),},
  {C_RGB(0,20,0),C_RGB(0,31,0),C_RGB(0,14,0),C_RGB(0,5,0),C_RGB(0,10,0),},
  {C_RGB(0,0,20),C_RGB(0,0,31),C_RGB(0,0,14),C_RGB(0,0,5),C_RGB(0,0,10),},

  {C_RGB(20,10,0),C_RGB(31,15,0),C_RGB(14,7,0),C_RGB(5,4,0),C_RGB(10,7,0),},
  {C_RGB(20,0,10),C_RGB(31,0,15),C_RGB(14,0,7),C_RGB(5,0,4),C_RGB(10,0,7),},
  {C_RGB(20,10,10),C_RGB(31,15,15),C_RGB(14,7,7),C_RGB(5,4,4),C_RGB(10,7,7)},

  {C_RGB(10,20,0),C_RGB(15,31,0),C_RGB(7,14,0),C_RGB(4,5,0),C_RGB(7,10,0)},
  {C_RGB(0,20,10),C_RGB(0,31,15),C_RGB(0,14,7),C_RGB(0,5,4),C_RGB(0,10,7)},
  {C_RGB(10,20,10),C_RGB(15,31,15),C_RGB(7,14,7),C_RGB(4,5,4),C_RGB(7,10,7)},

  {C_RGB(0,10,20),C_RGB(0,15,31),C_RGB(0,7,14),C_RGB(0,4,5),C_RGB(0,7,10)},
  {C_RGB(10,0,20),C_RGB(15,0,31),C_RGB(7,0,14),C_RGB(4,0,5),C_RGB(7,0,10)},
  {C_RGB(10,10,20),C_RGB(15,15,31),C_RGB(7,7,14),C_RGB(4,4,5),C_RGB(7,7,10)},
};


void cal_pal(){
  int i;
  for (i=PAL_PLAYER;i<(PLAYER_NUM+PAL_PLAYER);i++){
    pal_vf[i] = tank_cols[i-PAL_PLAYER].c2;
  }

  for (i=PAL_T;i<(PAL_T+PAL_T_S);i++){
    //    pal_vf[i] = C_RGB(minmaxrndint(0,10),minmaxrndint(0,10),minmaxrndint(0,4));
    int r,v,b;
    r = 20-18*(i-PAL_T)/PAL_T_S;
    v = 15+11*(i-PAL_T)/PAL_T_S;
    b = 6;//minmaxrndint(5,8);
    pal_vf[i] = C_RGB(r,v,b);
  }
  
  for (i=PAL_FUMEE;i<(PAL_FUMEE+PAL_FUMEE_S);i++){
    int r = 10 + 20 * (i-PAL_FUMEE) / PAL_FUMEE_S;
    int v = r-minmaxrndint(0,1);
    int b = r-minmaxrndint(0,1);
    pal_vf[i] = C_RGB(r,v,b);
  }
  for (i=PAL_WATER;i<(PAL_WATER+PAL_WATER_S);i++){
    int r,v,b;
    r = minmaxrndint(0,3);
    v = minmaxrndint(0,3);
    b = 10+21*(i-PAL_WATER)/PAL_WATER_S;
    pal_vf[i] = C_RGB(r,v,b);
  }
  for (i=PAL_SAND;i<(PAL_SAND+PAL_SAND_S);i++){
    int r,v,b;
    r = minmaxrndint(27,31);
    v = minmaxrndint(27,31);
    b = minmaxrndint(5,8);
    pal_vf[i] = C_RGB(r,v,b);
  }


}
void rotate_pal(){
  int i;
  int col=pal_vf[PAL_SAND];
  for (i=PAL_SAND;i<(PAL_SAND+PAL_SAND_S-1);i++){
    pal_vf[i] =  pal_vf[i+1];
  }
  pal_vf[PAL_SAND+PAL_SAND_S-1] =  col;
  slDMACopy((void*)S2D_PAL, (void*)NBG1_COL_ADR, (Uint32)S2D_PALSIZE);
}

int in_type(int a,int beg, int size){
  if (a<beg) return 0;
  if (a>=(beg+size)) return 0;
  return 1;
}

void fire_zone(int x, int y, int dx, int dy){
  static int c1=1;
  static int c2=1;
  static int c3=1;
  static int c4=1;


  Uint16 padp  = Smpc_Peripheral[0].push;
  Uint16 padd  = Smpc_Peripheral[0].data;
  
  int up =  !(padd & PER_DGT_KU);
  int down =  !(padd & PER_DGT_KD);
    
  if (!(padd & PER_DGT_TA)) {
    if (up) c1++;
    if (down) c1--;
  }
  if (!(padd & PER_DGT_TB)) {
    if (up) c2++;
    if (down) c2--;
  }
  if (!(padd & PER_DGT_TC)) {
    if (up) c3++;
    if (down) c3--;
  }
  if (!(padd & PER_DGT_TX)) {
    if (up) c4++;
    if (down) c4--;
  }


  //  slBMBoxFill(x,y,x+dx,y-dy,PAL_WATER);

  int i,j;
  for (i=0;i<dx;i++){
    bm_pixel_set(x+i,y,PAL_WATER+minmaxrndint(0,PAL_WATER_S-1));
  }
  
  for (j=1;j<dy;j++){
    for (i=0;i<dx;i++){
      int d1 = 1;
      int d2 = 1;
      int d3 = 1;
      int d4 = 1;
      
      Uint8 last = bm_pixel_get(x+i,y-j);
      last = PAL_WATER+(last-PAL_WATER)/2;
      //  if (!in_type(last,PAL_WATER,PAL_WATER_S)) last = PAL_WATER;
      Uint8 left = bm_pixel_get(x+i-1,y-(j-1));
      if (!in_type(left,PAL_WATER,PAL_WATER_S)) left = PAL_T;
      Uint8 under = bm_pixel_get(x+i,y-(j-1));
      if (!in_type(under,PAL_WATER,PAL_WATER_S)) under = PAL_T;
      Uint8 right = bm_pixel_get(x+i+1,y-(j-1));
      if (!in_type(right,PAL_WATER,PAL_WATER_S)) right = PAL_T;

      if (left == PAL_T) d1 = 0;
      if (under == PAL_T) d2 =0;
      if (right == PAL_T) d3 = 0;
      if (last == PAL_T) d4 = 0;


      Uint16 resu = (d1*c1*left+d2*c2*under+d3*c3*right+d4*c4*last)/(d1*c1+d2*c2+d3*c3+d4*c4);
      if (!in_type(resu,PAL_WATER,PAL_WATER_S)) resu = PAL_T;
      bm_pixel_set(x+i,y-j,resu);
    }
  }
  
  
  slBMBoxFill(x-40,y+10,x+40,y+16,PAL_T);

  for (i=0;i<c1;i++) bm_pixel_set(x+i,y+10,1);
  for (i=0;i<c2;i++) bm_pixel_set(x+i,y+11,2);
  for (i=0;i<c3;i++) bm_pixel_set(x+i,y+12,3);
  for (i=0;i<c4;i++) bm_pixel_set(x+i,y+13,4);



}



int terrain_check_borders(FIXED pos[XY]){
  int  ok = (((pos[X])<toFIXED(1.0*X_MAX/2)) 
	     && ((pos[X])>toFIXED(-1.0*X_MAX/2)) 
	     && ((pos[Y])<toFIXED(1.0*Y_MAX/2)) 
	     && ((pos[Y])>toFIXED(-1.0*Y_MAX/2)));
  return ok;
}

int terrain_check_type(FIXED from[XY],FIXED to[XY]){

  Uint8 from_ty = bm_pixel_get(from[X]>>16,from[Y]>>16);
  Uint8 to_ty = bm_pixel_get(to[X]>>16,to[Y]>>16);

  if (in_type(to_ty,PAL_WATER,PAL_WATER_S)){
    return 0;
  }
  return 1;
  
}

void test_palette(){

  slBMBoxFill(-X_MAX/2,-Y_MAX/2,X_MAX/2,Y_MAX/2 ,0);
  
  int i;
  for (i=0;i<256;i++){
    slBMLine(i-X_MAX/2+10,100,i-X_MAX/2+10,110,i);
  }
  int beg,end,y,dy,x1,x2;
  x1 = -150;
  x2 = -30;
  dy = 30;
  y = -100;
  
  beg = PAL_PLAYER;
  end = beg+PLAYER_NUM-1;
  y+=dy;
  text_write(bmNBG1,x1,y,"PAL PLAYER",1,1);
  for (i=beg;i<end;i++){
    slBMLine(x2+i,y,x2+i,y+10,i);
  }
    
  beg = PAL_T;
  end = beg+PAL_T_S-1;
  y+=dy;
  text_write(bmNBG1,x1,y,"PAL TERRAIN",1,1);
  for (i=beg;i<end;i++){
    slBMLine(x2+i,y,x2+i,y+10,i);
  }
    
  beg = PAL_FUMEE;
  end = beg+PAL_FUMEE_S-1;
  y+=dy;
  text_write(bmNBG1,x1,y,"PAL FUMEE",1,1);
  for (i=beg;i<end;i++){
    slBMLine(x2+i,y,x2+i,y+10,i);
  }
    
  beg = PAL_WATER;
  end = beg+PAL_WATER_S-1;
  y+=dy;
  text_write(bmNBG1,x1,y,"PAL WATER",1,1);
  for (i=beg;i<end;i++){
    slBMLine(x2+i,y,x2+i,y+10,i);
  }
    
  beg = PAL_SAND;
  end = beg+PAL_SAND_S-1;
  y+=dy;
  text_write(bmNBG1,x1,y,"PAL SAND",1,1);
  for (i=beg;i<end;i++){
    slBMLine(x2+i,y,x2+i,y+10,i);
  }
  do {
    slSynch();
  } while (!(pad_someone_presses(PER_DGT_ST,1)));
}

void copy_portions(){

  Sint8 buffer[16*16];
  int x,y;
  static int which = 0;

  which++;
  
  int rcx = which%22;
  int rcy = which/22;
  
  if (rcy==15) which=0;
  
  //  int rcx = minmaxrndint(0,X_MAX/16);
  //  int rcy = minmaxrndint(0,Y_MAX/16);

  x = rcx*16-X_MAX/2;
  y = rcy*16-Y_MAX/2;

  slBMGet(x,y,x+15,y+15,buffer);
  int i;
  for (i=0;i<(16*16);i++){
    Uint8 col = buffer[i];
    col--;
    if (col<=PAL_WATER) col = PAL_WATER;
    /*
      if (in_type(col,PAL_T,PAL_T_S)){
      //    col = PAL_T+(col-PAL_T)/2;
      //    col = PAL_T;
      col = PAL_SAND+minmaxrndint(0,PAL_SAND_S-1);
      } else if (in_type(col,PAL_FUMEE,PAL_FUMEE_S)){
      col = minmaxrndint(PAL_T,PAL_T_S-1);
      } else if (in_type(col,PAL_WATER,PAL_WATER_S)){
      // col = PAL_PLAYER+2;
      } else if (in_type(col,PAL_SAND,PAL_SAND_S)){
      //    col = PAL_SAND+minmaxrndint(0,PAL_SAND_S-1);
      } else if (in_type(col,PAL_PLAYER,PLAYER_NUM-1)){
      //    col = minmaxrndint(PAL_T,PAL_T_S-1);
      }
    */
    buffer[i] = col;
  }
  slBMPut(x,y,x+15,y+15,buffer);
}

void clear_one(){
  int x,y;
  x = minmaxrndint(-X_MAX/2,X_MAX/2);
  y = minmaxrndint(-Y_MAX/2,Y_MAX/2);

  Uint16 col = bm_pixel_get(x,y);
  if (in_type(col,PAL_T,PAL_T_S)){
    //    col = PAL_T+(col-PAL_T)/2;
    //    col = PAL_T;
    //    col = PAL_SAND+minmaxrndint(0,PAL_SAND_S-1);
  } else if (in_type(col,PAL_FUMEE,PAL_FUMEE_S)){
    col = minmaxrndint(PAL_T,PAL_T_S-1);
  } else if (in_type(col,PAL_WATER,PAL_WATER_S)){
    // col = PAL_PLAYER+2;
  } else if (in_type(col,PAL_SAND,PAL_SAND_S)){
    //    col = PAL_SAND+minmaxrndint(0,PAL_SAND_S-1);
  } else if (in_type(col,PAL_PLAYER,PLAYER_NUM-1)){
    //    col = minmaxrndint(PAL_T,PAL_T_S-1);
  }

  bm_pixel_set(x,y,col);

  int i;
  for (i=0;i<20;i++){
    x= x-5+minmaxrndint(0,5);
    y= y-5+minmaxrndint(0,5);

    Sint8  * where = slLocateBitMap(x,y);
    if (where){
      Uint8 col =  bm_pixel_get2(x,y);
      if (col== PAL_FUMEE){//&&(col<(PAL_FUMEE+3))){//+PAL_FUMEE_S-1){
	bm_pixel_set2(x,y,0);
      } else  if (in_type(col,PAL_FUMEE,PAL_FUMEE_S-1)){
	bm_pixel_set2(x,y,PAL_FUMEE+(col-PAL_FUMEE)/2);
      } else  if (in_type(col,PAL_PLAYER,PLAYER_NUM-1)){
	bm_pixel_set2(x,y,0);
      }       
    }
  }
}


void ground_destroy(int x,int y){
  Uint8 col = bm_pixel_get(x,y);
  
  /*  if (x>X_MAX/2) return;
      if (x<-X_MAX/2) return;
      if (y>Y_MAX/2) return;
      if (y<-Y_MAX/2) return;
  */

  if (in_type(col,PAL_T,PAL_T_S)){
    col = PAL_T + minmaxrndint(0,PAL_T_S-10);
  } else if (in_type(col,PAL_FUMEE,PAL_FUMEE_S)){
    col = PAL_T+minmaxrndint(0,PAL_T_S-1);
  } else if (in_type(col,PAL_WATER,PAL_WATER_S)){
    col = PAL_WATER+minmaxrndint(0,PAL_WATER_S-1);
  } else if (in_type(col,PAL_SAND,PAL_SAND_S)){
    //    col = PAL_SAND+minmaxrndint(0,PAL_SAND_S-1);
    col = PAL_SAND + minmaxrndint(0,PAL_SAND_S-1);
  }

  bm_pixel_set(x,y,col);
}

//
// Texte
//
typedef struct {
  Uint8 l1;
  Uint8 l2;
  Uint8 l3;
  Uint8 l4;
  Uint8 l5;
  Uint8 l6;
  Uint8 l7;
  Uint8 l8;

} TXT_CARAC;


TXT_CARAC carac_table[] = {

  {0,60,66,66,66,66,60,0}, //O
  {0,16,16,16,16,16,16,0}, //1
  {0,60,66,32,16,12,126,0}, //2
  {0,126,32,16,56,64,62,0}, //3
  {0,32,16,8,4,126,16,0}, //4
  {0,124,2,28,32,32,30,0}, // 5
  {0,2,2,30,34,34,30,0}, //6
  {0,126,32,56,16,8,4,0}, // 7
  {0,60,66,60,66,66,60,0}, //8
  {0,60,66,126,64,64,62,0}, //9


  {0,0,0,0,0,0,0,0},
  {0,0,0,0,0,0,0,0},
  {0,0,0,0,0,0,0,0},
  {0,0,0,0,0,0,0,0},

  {0,0,0,0,0,0,0,0},
  {0,0,0,0,0,0,0,0},
  {0,0,0,0,0,0,0,0},



  {0,126,66,126,66,66,66,0}, //A
  {0,126,66,62,66,66,126,0}, //B
  {0,126,2,2,2,2,126,0},     //C
  {0,62,66,66,66,66,62},     //D
  {0,126,2,30,2,2,126,0},    //E
  {0,126,2,30,2,2,2,0},      //F
  {0,126,2,122,66,66,126,0}, //G
  {0,66,66,126,66,66,66,0},  //H
  {0,126,16,16,16,16,126,0}, //I
  {0,126,32,32,32,32,24,0},  //J
  {0,18,10,6,10,18,34,0},    //K
  {0,2,2,2,2,2,126,0},       //L
  {0,66,66+36,66+24,66,66,66,0}, //M
  {0,66,66+4,66+8,66+16,66+32,66,0}, //N
  {0,60,66,66,66,66,60,0}, //O
  {0,126,66,126,2,2,2,0},  //P
  {0,60,66,66,66,66+32,126,0}, //Q
  {0,126,66,126,18,34,66,0}, //R
  {0,124,2,62,64,64,62,0}, //S
  {0,126,16,16,16,16,16,0}, // T
  {0,66,66,66,66,66,60,0},  // U
  {0,66,66,66,66,36,24,0}, // V
  {0,66,66,66,66+16,66+40,66+34,0}, //W
  {0,66,36,24,24,36,66,0}, //X
  {0,66,66,36,24,16,16,0}, // Y
  {0,126,32,16,8,4,126,0}, // Z
};

int text_write_char(Uint8 scroll, int x,int y,int num,int scale, int col){
  int i,j;
  int xx,yy;

  for (i=0;i<8;i++){
    for (j=0;j<8;j++){
      int content;
      switch (j) {
      case 0 : content = carac_table[num].l1; break;
      case 1 : content = carac_table[num].l2; break;
      case 2 : content = carac_table[num].l3; break;
      case 3 : content = carac_table[num].l4; break;
      case 4 : content = carac_table[num].l5; break;
      case 5 : content = carac_table[num].l6; break;
      case 6 : content = carac_table[num].l7; break;
      case 7 : content = carac_table[num].l8; break;
      }
      if (content & (1<<i)){
	int rcol = col;


	int n,m;
	for (n=0;n<scale;n++){
	  for (m=0;m<scale;m++){
	    if (col==-1){
	      rcol = PAL_T+minmaxrndint(0,PAL_WATER_S-1);
	      /*	      static int fcol = PAL_T;
	      fcol++;
	      if (fcol>(PAL_T+PAL_T_S-1)) fcol = PAL_T;
	      rcol = fcol;
	      */
	    }
	    xx= x+scale*i+n;
	    yy= y+scale*j+m;
	    bm_pixel_set_scroll(scroll,xx,yy,rcol);
	  }
	}
      }
    }
  }
  return (xx);
}



int text_write(Uint8 scroll,int x,int y,char * string, int scale, int col){
  char carac;
  int i=0;
  int xpos = x;
  for (;;){
    carac = string[i];
    if (carac == '\0')
      return (xpos);
    if (carac != ' '){
      xpos = 1+text_write_char(scroll,xpos,y,carac-48,scale,col);
    } else {
      xpos = xpos+3;
    }
    i++;
  }
  return xpos;
}

int text_write_int(Uint8 scroll,int x, int y, int num, int scale, int col){
  
  char istr[16];
  sprintf(istr,"%d",num);  
  return text_write(scroll,x,y,istr,scale,col);

}

//
// Terrain
//
#define INTERP_S 8
INTERPOINT Interp[INTERP_S];
int interp_s;

#define RANGE (PAL_WATER_S+PAL_SAND_S+PAL_T_S)

void randomize_interpoint(){
  //  int prop = 2;

  interp_s = INTERP_S;
  int xmin = -X_MAX/2;
  int xmax = X_MAX/2;
  int ymin = -Y_MAX/2;
  int ymax = Y_MAX/2;
  
  int n_groups = 4;// interp_s;//2;minmaxrndint(1,interp_s/4);

  int i = 0;
  do {
    int gx = minmaxrndint(xmin,xmax);
    int gy = minmaxrndint(ymin,ymax);
    int j;
    int spread = minmaxrndint(2,8);
    int dgxmin = -X_MAX/(2*spread);
    int dgxmax = X_MAX/(2*spread);
    int dgymin = -Y_MAX/(2*spread);
    int dgymax = Y_MAX/(2*spread);
    
    for (j=0;j<(interp_s/n_groups);j++){
      if (j==0) {
	Interp[i].size = 30000;
      } else {
	Interp[i].size = 10000;
      }
      Interp[i].prof = RANGE;
      //      Interp[i].size = minmaxrndint(10000,40000);

      Interp[i].x = gx + minmaxrndint(dgxmin,dgxmax);
      Interp[i].y = gy + minmaxrndint(dgymin,dgymax);
      i++;
    }
  } while (i<((interp_s/n_groups)*n_groups));
}


int Heightdapter[RANGE];
void heightdapter_init(){
  int i;
  for (i=0;i<RANGE;i++){
    ANGLE a = DEGtoANG((i*90.0/RANGE));
    FIXED mul = slSin(a);
    Heightdapter[i] = (int) ((slMulFX(toFIXED(1.0*RANGE),mul))>>16);
  }
  for (i=0;i<RANGE;i++){
    int val = Heightdapter[i];
    ANGLE a = DEGtoANG((val*90.0/RANGE));
    FIXED mul = slSin(a);
    Heightdapter[i] = (int) ((slMulFX(toFIXED(1.0*RANGE),mul))>>16);
  }
}

Sint8 calc_point(x,y){
  int k;
  Sint32 prof = 0;

  for (k=0;k<interp_s;k++){
    Sint32 dist = dist_int(x,y,Interp[k].x,Interp[k].y);
    if (dist!=-1){
      if (dist<=Interp[k].size){
	int n_prof;
	n_prof = Interp[k].prof - Interp[k].prof*dist/Interp[k].size;
	//	prof += Heightdapter[n_prof];
	prof += n_prof;
      } else {
	//	prof += PAL_WATER_S;
      }
    }
  }
  
  prof = prof/interp_s;
  
  Uint8 col;
  if (prof>=RANGE){
    col = PAL_T+minmaxrndint(0,PAL_T_S-1);
  } else {
    col = PAL_WATER+prof;
    if (in_type(col,PAL_SAND,PAL_SAND_S)){
      col = PAL_SAND+minmaxrndint(0,PAL_SAND_S-1);
    } else if (in_type(col,PAL_T,PAL_T_S)){
      col = col -4 + minmaxrndint(0,4);
    }
  }

  return col;
}
  

void display_progress_bar(int cur, int max){

  int xmin = -20;
  int xmax = 20;

  int step = (xmax-xmin)/max;

  int rxmin = xmin + cur*step;
  int rxmax = rxmin+step;

  int x;
  for (x=rxmin;x<rxmax;x++){
    int y;
    for (y=-10;y<10;y++){
      int col=1;
      bm_pixel_set2(x,y,col);    
    }
  }
}

void wait_message_on(){
  text_write(bmNBG0,-58,20,"PLEASE",5,PAL_FUMEE);
  text_write(bmNBG0,-48,60,"WAIT",5,PAL_FUMEE);
}
void wait_message_off(){
  int i,j;
  for (i=-48;i<48;i++){
    for (j=20;j<40;j++){
      bm_pixel_set_scroll(bmNBG0,i,j,0);
    } 
  }
}
void terrain_init(){

  heightdapter_init();
  // slZoomNbg1(toFIXED(2.0),toFIXED(2.0));
  
  
  
  slBMBoxFill(-X_MAX/2,-Y_MAX/2,X_MAX/2,Y_MAX/2 ,PAL_T);

 
  slScrAutoDisp(NBG0ON);
  wait_message_on();
  

  randomize_interpoint();
  int xmin = -X_MAX/2 +1;
  int xmax =  X_MAX/2 -1;
  int ymin = -Y_MAX/2+1;
  int ymax = Y_MAX/2 -1;
  int x,y;
  int pass;
  
  for (pass=0;pass<4;pass++){

    //display_progress_bar(pass,4);


    int xoffset = ((pass==0)||(pass==2))?0:1;
    int yoffset = ((pass==0)||(pass==1))?0:1;

    for (x=xmin+xoffset;x<xmax;x+=2){
      for (y=ymin+yoffset;y<ymax;y+=2){
	bm_pixel_set(x,y,calc_point(x,y));
      }
    }  
  }
  wait_message_off();

  int i;
  Uint16 temp_pal[256];
  for (i=0;i<256;i++){
    int tr = 0;//minmaxrndint(0,9);
    int tv = 0;//minmaxrndint(0,9);
    int tb = 0;//minmaxrndint(0,15);
    temp_pal[i] = C_RGB(tr,tv,tb);
  }
  slDMACopy((void*)temp_pal, (void*)NBG1_COL_ADR, (Uint32)S2D_PALSIZE);


 
  slScrAutoDisp(NBG1ON|NBG0ON|SPRON);


  int tourn;
  int n_tourn = 32;
  for (tourn=0;tourn<n_tourn;tourn++){
    for (i=0;i<256;i++){
      int r,v,b,tr,tv,tb;
      r = (pal_vf[i])&(0x1f);
      v = (pal_vf[i]>>5)&(0x1f);
      b = (pal_vf[i]>>10)&(0x1f);

      tr = r - (n_tourn-tourn);
      tv = v - (n_tourn-tourn);
      tb = b - (n_tourn-tourn);

      tr = (tr>0)?tr:0;
      tv = (tv>0)?tv:0;
      tb = (tb>0)?tb:0;

      temp_pal[i] = C_RGB(tr,tv,tb);
    }
    slSynch();
    slSynch();

    /*    do {
	  slSynch();
	  } while (!pad_someone_presses(PER_DGT_ST,1));
    */
    slDMACopy((void*)temp_pal, (void*)NBG1_COL_ADR, (Uint32)S2D_PALSIZE);
  }
}
