#include <sddrvs.dat>
#include "sgl.h"
#include "sound.h"

#include "pcm/b_eau.c"
#include "pcm/b_fire.c"
#include "pcm/b_tank.c"
#include "pcm/b_terre.c"
#include "pcm/m_tank.c"
#include "pcm/m_tire.c"
#include "pcm/s_tire.c"

#define WSOUND 1

extern int pad_someone_presses(Uint16,int);

static Uint32 soffset = 0x0;//0x2C;
SOUND_PCM vs_sound[vs_last];
#define SOUND_N_CHANNEL 4
SOUND_CHANNEL_STATE channel_state[SOUND_N_CHANNEL];

void channel_state_init(){
  int i;
  for (i=0;i<SOUND_N_CHANNEL;i++){
    channel_state[i].type = -1;
    channel_state[i].prio = -1;
    channel_state[i].age = 0;
  }
}

void sound_init()
{
#ifdef WSOUND
  char sound_map[] = {0xff , 0xff} ;
  slInitSound(sddrvstsk , sizeof(sddrvstsk) , (Uint8 *)sound_map , sizeof(sound_map)) ;
  slSndVolume(127);
  sound_init_sounds();
  channel_state_init();
#endif
}


void sione(SOUND_TYPE st, char* d, Uint32 s, PCM* info){
#ifdef WSOUND
  vs_sound[st].data = d+soffset;
  vs_sound[st].size = s-soffset;
  vs_sound[st].info  = info;
#endif
}

void sound_init_sounds(){  
#ifdef WSOUND
  sione(vs_b_eau,b_eau,b_eau_size,&b_eau_dat);
  sione(vs_b_fire,b_fire,b_fire_size,&b_fire_dat);
  sione(vs_b_tank,b_tank,b_tank_size,&b_tank_dat);
  sione(vs_b_terre,b_terre,b_terre_size,&b_terre_dat);
  sione(vs_m_tank,m_tank,m_tank_size,&m_tank_dat);
  sione(vs_m_tire,m_tire,m_tire_size,&m_tire_dat);
  sione(vs_s_tire,s_tire,s_tire_size,&s_tire_dat);
#endif
}

void sound_channel_print(int x, int y){
#ifdef WSOUND
  int i;
  for (i=0;i<SOUND_N_CHANNEL;i++){
    if (channel_state[i].type!=-1){
      slPrintHex(channel_state[i].type,slLocate(x,y+i));
      slPrintHex(channel_state[i].prio,slLocate(x+8,y+i));
      slPrintHex(channel_state[i].age ,slLocate(x+16,y+i));
    } else {
      slPrint("|              ",slLocate(x,y+i));
      slPrint("               ",slLocate(x+8,y+i));
      slPrint("               ",slLocate(x+16,y+i));
    }
  }
#endif
}

void sound_step(){
#ifdef WSOUND
  int i;
  for (i=0;i<SOUND_N_CHANNEL;i++){
    channel_state[i].age +=1;
  }
  sound_channel_state_get();
#endif
}

void sound_channel_state_get(){
#ifdef WSOUND
  static PCM sndo_dat = {(_Mono | _PCM8Bit) , 0 , 127 , 0 , 0x69ce , 0,0,0,0};

  int i;
  for (i=0;i<SOUND_N_CHANNEL;i++){
    sndo_dat.channel = i;
    if (!slPCMStat(&sndo_dat))
      channel_state[i].type = -1;
  }
#endif
}

#define MIN_CUT_AGE 10

void sound_get_least_prio(int prio, int* chan){
#ifdef WSOUND

  int i;
  sound_channel_state_get();

  int candidate = -1;
  int age_candidate = -1;
  int minage  = -1;
  int minage_prio  = -1;


  for (i=0;i<SOUND_N_CHANNEL;i++){
    if (channel_state[i].type == -1){ // un canal est libre (ca ne devrait pas être le cas)
      *chan = i;
      return;
    }  else {                         // quel canal couper ?
      if (minage<channel_state[i].age) {
	minage = channel_state[i].age;
	age_candidate = i;
      }
      
      if (prio == channel_state[i].prio) {
	if (minage_prio<channel_state[i].age) {
	  minage_prio = channel_state[i].age;
	  candidate = i;
	}
      }
    }
  }
  if (minage > MIN_CUT_AGE){
    
    if (candidate != -1){
      *chan=candidate;
    } else {      
      *chan=age_candidate;
    }
  } else {
    *chan= -1;
  }

#endif
}

void sound_tell(SOUND_TYPE st, int prio){
#ifdef WSOUND
  PCM * dpcm = vs_sound[st].info;
  sound_tell_more(st,prio,dpcm->level,dpcm->pan,(dpcm->pitch));
#endif
}


void sound_test(){
#ifdef WSOUND
  int cur1=0;
  int cur2=0;
  
  int st=0;
  int prio=0;
  Uint8 level=127;
  Sint8 pan = 0;
  Uint16 pitch = 0x69ce;

  for (;;){

    sound_channel_print(10,20);
    sound_channel_state_get();
    sound_step();

    cur2=0;

    if (pad_someone_presses(PER_DGT_KU, TRUE)) cur1--;
    if (pad_someone_presses(PER_DGT_KD, TRUE)) cur1++;
    if (pad_someone_presses(PER_DGT_KL, TRUE)) cur2=-1;
    if (pad_someone_presses(PER_DGT_KR, TRUE)) cur2=1;

    int y=5;
    if (cur1<0) cur1=0;
    if (cur1>4) cur1=4;

    if (cur1==0){
      prio += cur2;
    } else if (cur1==1){
      level+= cur2;
    } else if (cur1==2){
      pan +=cur2;
    } else if (cur1==3){
      pitch +=cur2;
    } else if (cur1==4){
      st +=cur2;
    }

    slPrint(" ",slLocate(1,y));
    slPrint("prio",slLocate(3,y));
    slPrintHex(prio,slLocate(20,y++));

    slPrint(" ",slLocate(1,y));
    slPrint("level",slLocate(3,y));
    slPrintHex(level,slLocate(20,y++));

    slPrint(" ",slLocate(1,y));
    slPrint("pan",slLocate(3,y));
    slPrintHex(pan,slLocate(20,y++));

    slPrint(" ",slLocate(1,y));
    slPrint("pitch",slLocate(3,y));
    slPrintHex(pitch,slLocate(20,y++));

    slPrint(" ",slLocate(1,y));
    slPrint("type",slLocate(3,y));

    slPrint(">",slLocate(1,5+cur1));
 
    if (pad_someone_presses(PER_DGT_TA, TRUE))
      sound_tell_more(st,prio,level,pan,pitch);



    
    slSynch();
  }
#endif

}

void sound_tell_more(SOUND_TYPE st, int prio, Uint8 level, Sint8 pan, Uint16 pitch){
#ifdef WSOUND
  
  PCM * dpcm = vs_sound[st].info;
  void* laddr = vs_sound[st].data;
  Uint32 lsize = vs_sound[st].size;
  PCM lpcm;
  
  if (st==vs_none) return;

  lpcm.mode    = dpcm->mode ;
  lpcm.channel = dpcm->channel;
  lpcm.pan     = pan;

  lpcm.pitch   = pitch;
  lpcm.level   = level;

  int channel  =  slPCMOn(&lpcm,laddr,lsize);
  //The return value is 0 to 7 for normal completion, -1 when there is not enough 
  //room in the command buffer, -2 when there is no PCM channel available, and -3 
  //when there is not enough room in the PCM buffer.

  if (channel == 1) return;
  if (channel == -3) return;
  
  if (channel>=0){
    channel_state[channel].type = st;
    channel_state[channel].prio = prio;
    channel_state[channel].age = 0;
  } else {
    int cut;
    sound_get_least_prio(prio, &cut);
    if (cut!=-1){
      lpcm.channel = cut;
      slPCMOff(&lpcm);
      sound_tell_more(st, prio,level,pan,pitch);      
    }
  }
#endif
}

void sound_pcm_all_off(){
#ifdef WSOUND
  PCM lpcm;
  int i;
  for (i=0;i<SOUND_N_CHANNEL;i++){
    lpcm.channel = i;
    slPCMOff(&lpcm);
  }
  sound_channel_state_get();
#endif
}

void sound_pcm_gfx_display(SOUND_TYPE st,int x1, int y1, int x2, int y2){
#ifdef WSOUND
  
  int i;

  slBMBoxFill(x1,y1,x2,y2,0);

  int dx = x2-x1;
  int dy = y2-y1;
  int ycenter = y1+dy/2;

  for (i=0;i<dx;i++){
    Uint8* add = (Uint8*) ((vs_sound[st].data + (vs_sound[st].size*i)/dx));
    Sint8 s = *add;
    int col = (int)add%256;
    slBMLine(x1+i,ycenter,x1+i,ycenter+s*dy/128,col);
  }
#endif
  
}

